@@ -14,6 +14,7 @@ import posixpath
 import re
 import os
 import codecs
+import traceback
 try:
     from hashlib import sha1 as sha
 except ImportError:
@@ -26,11 +27,14 @@ from sphinx.errors import SphinxError
 from sphinx.util.osutil import ensuredir, ENOENT, EPIPE
 from sphinx.util.compat import Directive
 
-from actdiag_sphinxhelper import command, diagparser, builder, DiagramDraw
+from actdiag_sphinxhelper import command, parser, builder, drawer
 from actdiag_sphinxhelper import collections, FontMap
 from actdiag_sphinxhelper import actdiag, ActdiagDirective
 namedtuple = collections.namedtuple
 
+import blockdiag_sphinxhelper
+detectfont = blockdiag_sphinxhelper.command.detectfont
+
 
 class ActdiagError(SphinxError):
     category = 'Actdiag error'
@@ -40,7 +44,7 @@ class Actdiag(ActdiagDirective):
     def run(self):
         try:
             return super(Actdiag, self).run()
-        except diagparser.ParseException, e:
+        except parser.ParseException, e:
             if self.content:
                 msg = '[%s] ParseError: %s\n%s' % (self.name, e, "\n".join(self.content))
             else:
@@ -57,10 +61,10 @@ def get_image_filename(self, code, format, options, prefix='actdiag'):
     """
     Get path of output file.
     """
-    if format not in ('PNG', 'PDF', 'SVG'):
+    if format.upper() not in ('PNG', 'PDF', 'SVG'):
         raise ActdiagError('actdiag error:\nunknown format: %s\n' % format)
 
-    if format == 'PDF':
+    if format.upper() == 'PDF':
         try:
             import reportlab
         except ImportError:
@@ -108,7 +112,7 @@ def get_fontmap(self):
 
         if fontpath:
             config = namedtuple('Config', 'font')(fontpath)
-            _fontpath = command.detectfont(config)
+            _fontpath = detectfont(config)
             fontmap.set_default_font(_fontpath)
     except:
         attrname = '_actdiag_fontpath_warned'
@@ -125,7 +129,7 @@ def get_anchor(self, refid, fromdocname):
     for docname in self.builder.env.found_docs:
         doctree = self.builder.env.get_doctree(docname)
         for target in doctree.traverse(nodes.Targetable):
-            if target.attributes['refid'] == refid:
+            if target.attributes.get('refid') == refid:
                 targetfile = self.builder.get_relative_uri(fromdocname, docname)
                 return targetfile + "#" + refid
 
@@ -148,7 +152,7 @@ def create_actdiag(self, code, format, filename, options, prefix='actdiag'):
     draw = None
     fontmap = get_fontmap(self)
     try:
-        tree = diagparser.parse(diagparser.tokenize(code))
+        tree = parser.parse_string(code)
         screen = builder.ScreenNodeBuilder.build(tree)
 
         for lane in screen.lanes:
@@ -159,9 +163,12 @@ def create_actdiag(self, code, format, filename, options, prefix='actdiag'):
                 node.href = resolve_reference(self, node.href, options)
 
         antialias = self.builder.config.actdiag_antialias
-        draw = DiagramDraw.DiagramDraw(format, screen, filename,
-                                       fontmap=fontmap, antialias=antialias)
+        draw = drawer.DiagramDraw(format, screen, filename,
+                                  fontmap=fontmap, antialias=antialias)
     except Exception, e:
+        if self.builder.config.actdiag_debug:
+            traceback.print_exc()
+
         raise ActdiagError('actdiag error:\n%s\n' % e)
 
     return draw
@@ -196,7 +203,7 @@ def make_imgtag(self, image, relfn, trelfn, outfn,
 
     if clickable_map:
         imgtag_format = '<img src="%s" alt="%s" width="%s" '
-        imgtag_format += 'usemap="#map_1" height="%s" />\n'  # TODO:mapname
+        imgtag_format += 'usemap="#map_%d" height="%%s" />\n' % id(image)
     else:
         imgtag_format = '<img src="%s" alt="%s" width="%s" height="%s" />\n'
 
@@ -210,7 +217,7 @@ def make_imgtag(self, image, relfn, trelfn, outfn,
                    (relfn, alt, image_size[0], image_size[1]))
 
     if clickable_map:
-        result += ('<map name="map_1">')
+        result += ('<map name="map_%d">' % id(image))
         rect_format = '<area shape="rect" coords="%s,%s,%s,%s" href="%s">'
         for m in clickable_map:
             x1 = m[0][0]
@@ -249,7 +256,6 @@ def render_dot_html(self, node, code, options, prefix='actdiag',
             thumb_size = (options['maxwidth'], image_size[1] * ratio)
             if not os.path.isfile(toutfn):
                 image.filename = toutfn
-                image.draw()
                 image.save(thumb_size)
 
     except UnicodeEncodeError, e:
@@ -268,7 +274,7 @@ def render_dot_html(self, node, code, options, prefix='actdiag',
         if alt is None:
             alt = node.get('alt', self.encode(code).strip())
 
-        if format == 'SVG':
+        if format.upper() == 'SVG':
             tagfunc = make_svgtag
         else:
             tagfunc = make_imgtag
@@ -308,7 +314,7 @@ def latex_visit_actdiag(self, node):
 
 
 def on_doctree_resolved(self, doctree, docname):
-    if self.builder.name in ('gettext', 'singlehtml', 'html', 'latex'):
+    if self.builder.name in ('gettext', 'singlehtml', 'html', 'latex', 'epub'):
         return
 
     for node in doctree.traverse(actdiag):
@@ -336,6 +342,7 @@ def setup(app):
     app.add_config_value('actdiag_fontpath', None, 'html')
     app.add_config_value('actdiag_fontmap', None, 'html')
     app.add_config_value('actdiag_antialias', False, 'html')
+    app.add_config_value('actdiag_debug', False, 'html')
     app.add_config_value('actdiag_html_image_format', 'PNG', 'html')
     app.add_config_value('actdiag_tex_image_format', 'PNG', 'html')
     app.connect("doctree-resolved", on_doctree_resolved)
